/*
 * Swiss-Phong shader.
 *
 * This is a surface shader which outputs a Blinn-Phong shaded 
 * image optionally using ambient occlusion.  Additionally,
 * many output variables are set to split out different light
 * groups.
 * 
 * (c) 2007 Jeremy Huddleston <jeremyhu@cs.berkeley.edu>
 *
 * Released under the terms of the LGPL-2:
 * http://www.fsf.org/licensing/licenses/lgpl.txt
 */

color specular_indexed(normal Nn;
                       vector V;
                       float roughness;
                       output color specular_light_array[8];) {
    color C = 0;
    extern point P;

    illuminance (P, Nn, PI/2) {
        uniform float id=0;
        lightsource ("group_id", id);

        /* Blinn's hack of Phong's hack */
        vector H = normalize(L) + normalize(V);

        float HdotN = (normalize(H) . Nn);

        if(HdotN > 0) {
            color currentColor = Cl * pow(HdotN, 1 / roughness);

            C += currentColor;
            specular_light_array[id] += currentColor;

        }
    }

    return C;
}

color diffuse_indexed(normal Nn; output color diffuse_light_array[8];) {
    color C = 0;
    extern point P;

    illuminance (P, Nn, PI/2) {
        uniform float id=0;
        lightsource ("group_id", id);

        color currentColor = Cl * (normalize(L) . Nn);

        C += currentColor;
        diffuse_light_array[id] += currentColor;
    }

    return C;
}

surface swiss_phong(float Ka = 1;
                    float ambocclSamples = 0;
                    float ambocclMethod = 0;
                    float ambocclMaxDist = 1e30;
                    float ambocclConeAngle = PI/2;

                    float Kd = 1;

                    float Ks = 1;
                    float roughness = .1;
                    color specularColor = 1;

                    output varying color amboccl = 1;
                    output varying color light_ambient = 0;                    

                    output varying color light_diffuse_0 = 0;
                    output varying color light_diffuse_1 = 0;
                    output varying color light_diffuse_2 = 0;
                    output varying color light_diffuse_3 = 0;
                    output varying color light_diffuse_4 = 0;
                    output varying color light_diffuse_5 = 0;
                    output varying color light_diffuse_6 = 0;
                    output varying color light_diffuse_7 = 0;
                    output varying color light_specular_0 = 0;
                    output varying color light_specular_1 = 0;
                    output varying color light_specular_2 = 0;
                    output varying color light_specular_3 = 0;
                    output varying color light_specular_4 = 0;
                    output varying color light_specular_5 = 0;
                    output varying color light_specular_6 = 0;
                    output varying color light_specular_7 = 0;) {

    color diffuse_light_array[8] = {0, 0, 0, 0, 0, 0, 0, 0};
    color specular_light_array[8] = {0, 0, 0, 0, 0, 0, 0, 0};

    normal Nf = faceforward(normalize(N), I);
    vector V = -normalize(I);
    float occ = 0;

    if(ambocclSamples > 0) {
        if(ambocclMethod == 0) {
            occ = occlusion(P, Nf, ambocclSamples,
                            "maxdist", ambocclMaxDist, "coneangle", ambocclConeAngle,
                            "distribution", "cosine");
        } else {
            float hits = 0;
            gather("illuminance", P, Nf, ambocclConeAngle, ambocclSamples,
                   "distribution", "cosine", "maxdist", ambocclMaxDist) {
                hits += 1;
            }
            occ = hits / ambocclSamples;
        }
    }
    amboccl = 1 - occ;
    light_ambient = Ka * ambient();

    Ci  = Cs * (amboccl * light_ambient + Kd * diffuse_indexed(Nf, diffuse_light_array));
    Ci += specularColor * Ks * specular_indexed(Nf, V, roughness, specular_light_array);

    Oi = Os;
    Ci *= Oi;

    light_diffuse_0 = Kd * diffuse_light_array[0];
    light_diffuse_1 = Kd * diffuse_light_array[1];
    light_diffuse_2 = Kd * diffuse_light_array[2];
    light_diffuse_3 = Kd * diffuse_light_array[3];
    light_diffuse_4 = Kd * diffuse_light_array[4];
    light_diffuse_5 = Kd * diffuse_light_array[5];
    light_diffuse_6 = Kd * diffuse_light_array[6];
    light_diffuse_7 = Kd * diffuse_light_array[7];
    light_specular_0 = Ks * specular_light_array[0];
    light_specular_1 = Ks * specular_light_array[1];
    light_specular_2 = Ks * specular_light_array[2];
    light_specular_3 = Ks * specular_light_array[3];
    light_specular_4 = Ks * specular_light_array[4];
    light_specular_5 = Ks * specular_light_array[5];
    light_specular_6 = Ks * specular_light_array[6];
    light_specular_7 = Ks * specular_light_array[7];
}
